#include "ball.h"
#include <QDebug>
#include <QRect>
#include <QWidget>
#include <qglobal.h>
#include <random>

#define RADIUS 15
#define TICK_SPEED 10
#define MAX_SPEED 5.0
#define GRAVITY 0.981

double getRandomDouble(double maxValue);

Ball::Ball(QWidget* parent, std::shared_ptr<Platform> platform) : m_parent{parent}, m_platform{platform}, m_isSpawning{true} {
    QRect rct = parent->rect();

    m_x = rand() % rct.width(); // random x pos
    m_y = rct.height() + 5; // start below the screen

    m_kx = getRandomDouble(MAX_SPEED); // random horizontal speed
    m_ky = -MAX_SPEED;

    start(TICK_SPEED);
}

void Ball::draw(QPainter& painter) {
    painter.drawEllipse(m_x, m_y, RADIUS * 2.0, RADIUS * 2.0);
}

void Ball::timerEvent(QTimerEvent*) {
    if (m_ky < -MAX_SPEED)
        m_ky = std::max(m_ky - GRAVITY, -MAX_SPEED);

    // Horizontal wall collision
    QRect rct = m_parent->rect();
    if (m_x < RADIUS || m_x > rct.width() - RADIUS) {
        m_kx *= -1;
    }
    // Ceiling collision
    if (m_y < RADIUS) {
        m_ky *= -1;
    }
    // Bottom collision (only if no longer spawning)
    if (!m_isSpawning && m_y > rct.height() - RADIUS) {
        emit deleted(this);
        stop();
        m_parent->update();
        return;
    }

    // Platform collision (only if no longer spawning)
    if (!m_isSpawning && m_platform->rect().intersects(QRect(m_x, m_y, RADIUS * 2.0, RADIUS * 2.0))) {
        m_ky *= -1;
    }

    m_x += m_kx;
    m_y += m_ky;

    // Once we get above half point of the window, we're no longer "spawning"
    if (m_isSpawning && m_y < rct.height() / 2.0 - RADIUS) {
        m_isSpawning = false;
    }

    // Update the window
    m_parent->update();
}

double getRandomDouble(double maxValue) {
    // Create a random device and a random engine
    std::random_device rd;
    std::mt19937 gen(rd());

    // Define the range from 0 to maxValue
    std::uniform_real_distribution<> dis(0.0, maxValue);

    // Return the random value
    return dis(gen);
}
