#include "mainwindow.h"
#include "./ui_mainwindow.h"
#include <QDebug>
#include <memory>
#include <qnamespace.h>

MainWindow::MainWindow(QWidget* parent) : QMainWindow(parent), ui(new Ui::MainWindow), leftArrowPressed{false}, rightArrowPressed{false} {
    ui->setupUi(this);

    platform = std::make_shared<Platform>(this);

    auto ball = std::make_shared<Ball>(this, platform);
    connect(ball.get(), &Ball::deleted, this, &MainWindow::onDeleteBall);
    balls.append(ball);
}

MainWindow::~MainWindow() {
    delete ui;
}

void MainWindow::paintEvent(QPaintEvent*) {
    QPainter painter(this);

    painter.fillRect(rect(), Qt::darkGray);

    painter.setBrush(QBrush(Qt::white));
    painter.setPen(Qt::black);

    for (auto ball : balls) {
        ball->draw(painter);
    }

    platform->draw(painter);
}

void MainWindow::onDeleteBall(Ball* ball) {
    // Remove the ball from the list by checking the raw pointer
    // This will also free it (smart ptrs).
    for (auto it = balls.begin(); it != balls.end(); ++it) {
        if (it->get() == ball) {
            qDebug() << "Deleting ball";
            balls.erase(it);
            break;
        }
    }
}

void MainWindow::keyPressEvent(QKeyEvent* event) {
    switch (event->key()) {
        case Qt::Key_Left: leftArrowPressed = true; break;
        case Qt::Key_Right: rightArrowPressed = true; break;
    }
}

void MainWindow::keyReleaseEvent(QKeyEvent* event) {
    switch (event->key()) {
        case Qt::Key_Left: leftArrowPressed = false; break;
        case Qt::Key_Right: rightArrowPressed = false; break;
    }
}
